# Test CawtExcel procedures to exchange data between Excel and RAW photo images.
#
# Copyright: 2007-2012 Paul Obermeier (obermeier@poSoft.de)
# Distributed under BSD license.

set cawtDir [file join [pwd] ".."]
set auto_path [linsert $auto_path 0 $cawtDir [file join $cawtDir "Externals"]]

package require cawt

set rawFile "testIn/gradient.raw"

set retVal [catch {package require img::raw} version]
if { $retVal == 0 } {
    set phImg [image create photo -file $rawFile \
               -format "RAW -useheader 1 -verbose 0 -nomap 0 -gamma 1"]
    pack [label .l] -side left
    .l configure -image $phImg
    wm title . "Original RAW image vs. generated RAW images \
               (Size: [image width $phImg] x [image height $phImg])"
    update
} else {
    puts "Package img::raw is not available."
    puts "Raw images will be read and written, but not displayed in a Tk window."
}

# Open new instance of Excel and add a workbook.
set excelAppId [::Excel::OpenNew]
set workbookId [::Excel::AddWorkbook $excelAppId]

# Delete Excel and RAW image files from previous test run.
file mkdir testOut
set xlsFile [file join [pwd] "testOut" "Excel-11_RawImage"]
append xlsFile [::Excel::GetExtString $excelAppId]
file delete -force $xlsFile
set rawOutFile1 [file join [pwd] "testOut" "Excel-11_RawImage1.raw"]
file delete -force $rawOutFile1
set rawOutFile2 [file join [pwd] "testOut" "Excel-11_RawImage2.raw"]
file delete -force $rawOutFile2

# Transfer image data with header information into Excel and vice versa.
set useHeader true

set worksheetId [::Excel::AddWorksheet $workbookId "WithHeader"] 

set t1 [clock clicks -milliseconds]
::Excel::RawImageFileToWorksheet $rawFile $worksheetId $useHeader
set t2 [clock clicks -milliseconds]
puts "RawImageFileToWorksheet: [expr $t2 - $t1] ms (using header: $useHeader)."

set t1 [clock clicks -milliseconds]
::Excel::WorksheetToRawImageFile $worksheetId $rawOutFile1 $useHeader
set t2 [clock clicks -milliseconds]
puts "WorksheetToRawImageFile: [expr $t2 - $t1] ms (using header: $useHeader)."


# Transfer image data without header information into Excel and vice versa.
set useHeader false

set worksheetId [::Excel::AddWorksheet $workbookId "NoHeader"] 

set t1 [clock clicks -milliseconds]
::Excel::RawImageFileToWorksheet $rawFile $worksheetId $useHeader
set t2 [clock clicks -milliseconds]
puts "RawImageFileToWorksheet: [expr $t2 - $t1] ms (using header: $useHeader)."

set t1 [clock clicks -milliseconds]
::Excel::WorksheetToRawImageFile $worksheetId $rawOutFile2 $useHeader
set t2 [clock clicks -milliseconds]
puts "WorksheetToRawImageFile: [expr $t2 - $t1] ms (using header: $useHeader)."

if { $retVal == 0 } {
    set phImg1 [image create photo -file $rawOutFile1 \
                -format "RAW -useheader 1 -verbose 0 -nomap 0 -gamma 1"]
    pack [label .l1] -side left
    .l1 configure -image $phImg1
    set phImg2 [image create photo -file $rawOutFile2 \
                -format "RAW -useheader 1 -verbose 0 -nomap 0 -gamma 1"]
    pack [label .l2] -side left
    .l2 configure -image $phImg2
}

puts "Saving as Excel file: $xlsFile"
::Excel::SaveAs $workbookId $xlsFile

if { [lindex $argv 0] eq "auto" } {
    ::Excel::Quit $excelAppId
    ::Cawt::Destroy
    exit 0
}
::Cawt::Destroy
